--- Mailman/Archiver/HyperArch.py.orig	Thu Apr 24 20:53:37 2003
+++ Mailman/Archiver/HyperArch.py	Mon Jun  9 19:24:37 2003
@@ -150,6 +150,13 @@
 # Blank lines
 blankpat = re.compile(r'^\s*$')
 
+# CVS mailing list
+cvslistpat = re.compile(r'^.*cvs.*$')
+# CVS commit mail subject
+cvspat = re.compile(r'^cvs commit[:]')
+# CVS Revision/Changes/Path line
+cvsrevpathpat = re.compile(r'^\s*(?P<rev>(?:\d+[.])+\d+)\s+[+]\d+\s+[-]\d+\s+(?P<path>\S+)\s*(?:[(](?P<state>[a-zA-Z]+)[)])?\s*$')
+
 # Starting <html> directive
 htmlpat = re.compile(r'^\s*<HTML>\s*$', re.IGNORECASE)
 # Ending </html> directive
@@ -1084,6 +1091,48 @@
                 d[each] = self.__dict__[each]
         return d
 
+    # Add <A HREF="..."> tags around cvs commit paths.
+
+    def __processbody_CVSquote(self, lines):
+        source = lines[:]
+        dest = lines
+        for i in xrange(0, len(source)):
+            Lorig = L = source[i]
+            if L is None:
+                continue
+            revpath = cvsrevpathpat.search(L)
+            if revpath:
+                pos = revpath.start('path')
+                if pos != 0:
+                    rev = revpath.group('rev')
+                    path = revpath.group('path')
+                    state = revpath.group('state')
+                    revs = rev.split('.')
+                    # Try to guess the 'previous' revision
+                    if revs[-1] != '0':
+                        revs[-1] = str(int(revs[-1]) - 1)
+                    if revs[-1] == '0':
+                        revs = revs[:-2]
+                    prevrev = '.'.join(revs)
+                    if state == 'dead':
+                        prevrev = ''
+                        pathparts = path.split('/')
+                        pathparts.insert(len(pathparts)-1, 'Attic')
+                        urlpath = '/'.join(pathparts)
+                    else:
+                        urlpath = path
+                    if prevrev:
+                        URL = '%s/%s.diff?r1=%s&r2=%s' % (
+                            mm_cfg.CVSWEB, html_quote(urlpath), prevrev, rev)
+                    else:
+                        URL = '%s/%s?rev=%s' % (
+                            mm_cfg.CVSWEB, html_quote(urlpath), rev)
+                    length = len(path)
+                    L = '%s<A HREF="%s">%s</A>%s' % (
+                        L[:pos], URL, path, L[pos+length:])
+            source[i] = None
+            dest[i] = L
+
     # Add <A HREF="..."> tags around URLs and e-mail addresses.
 
     def __processbody_URLquote(self, lines):
@@ -1193,6 +1242,15 @@
         if self.ALLOWHTML:
             self.__processbody_HTML(lines)
         self.__processbody_URLquote(lines)
+        try:
+            mm_cfg.CVSWEB
+        except AttributeError:
+            pass
+        else:
+            if mm_cfg.CVSWEB:
+                if cvslistpat.match(self.maillist.real_name) is not None:
+                    if cvspat.match(article.subject) is not None:
+                        self.__processbody_CVSquote(lines)
         if not self.SHOWHTML and lines:
             lines.insert(0, '<PRE>')
             lines.append('</PRE>')
